/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsIChannel.idl
 */

#ifndef __gen_nsIChannel_h__
#define __gen_nsIChannel_h__

#include "nsIRequest.h"

class nsIURI; /* forward declaration */

class nsIInputStream; /* forward declaration */

class nsIOutputStream; /* forward declaration */

class nsIStreamObserver; /* forward declaration */

class nsIStreamListener; /* forward declaration */

class nsILoadGroup; /* forward declaration */

class nsIInterfaceRequestor; /* forward declaration */

class nsIFile; /* forward declaration */

typedef PRUint32 nsLoadFlags;


/* starting interface:    nsIChannel */
#define NS_ICHANNEL_IID_STR "2d905684-8b5c-11d3-8cd9-0060b0fc14a3"

#define NS_ICHANNEL_IID \
  {0x2d905684, 0x8b5c, 0x11d3, \
    { 0x8c, 0xd9, 0x00, 0x60, 0xb0, 0xfc, 0x14, 0xa3 }}

/**
 * nsIChannel is the abstract base class for transports and URLs.
 * It's abstract in that it doesn't provide a means to specify the
 * location/destination of the data being accessed. 
 */
class nsIChannel : public nsIRequest {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_ICHANNEL_IID)

  /**
     * Returns the original URL used to construct the channel.
     */
  /* readonly attribute nsIURI originalURI; */
  NS_IMETHOD GetOriginalURI(nsIURI * *aOriginalURI) = 0;

  /**
     * Returns the URL to which the channel currently refers.
     */
  /* readonly attribute nsIURI URI; */
  NS_IMETHOD GetURI(nsIURI * *aURI) = 0;

  /**
     * Opens a blocking input stream to the URL's specified source.
     * @param startPosition - The offset from the start of the data
     *   from which to read.
     * @param readCount - The number of bytes to read. If -1, everything
     *   up to the end of the data is read. If greater than the end of 
     *   the data, the amount available is returned in the stream.
     */
  /* nsIInputStream openInputStream (in unsigned long startPosition, in long readCount); */
  NS_IMETHOD OpenInputStream(PRUint32 startPosition, PRInt32 readCount, nsIInputStream **_retval) = 0;

  /**
     * Opens a blocking output stream to the URL's specified destination.
     * @param startPosition - The offset from the start of the data
     *   from which to begin writing.
     */
  /* nsIOutputStream openOutputStream (in unsigned long startPosition); */
  NS_IMETHOD OpenOutputStream(PRUint32 startPosition, nsIOutputStream **_retval) = 0;

  /**
     * Opens the channel asynchronously. The nsIStreamObserver's OnStartRequest
     * method is called back when the channel actually becomes open, providing 
     * the content type. Its OnStopRequest method is called when the channel
     * becomes closed.
     */
  /* void asyncOpen (in nsIStreamObserver observer, in nsISupports ctxt); */
  NS_IMETHOD AsyncOpen(nsIStreamObserver *observer, nsISupports *ctxt) = 0;

  /**
     * Reads asynchronously from the URL's specified source. Notifications
     * are provided to the stream listener on the thread of the specified
     * event queue.
     * The startPosition argument designates the offset in the source where
     * the data will be read.
     * If the readCount == -1 then all the available data is delivered to
     * the stream listener.
     */
  /* void asyncRead (in unsigned long startPosition, in long readCount, in nsISupports ctxt, in nsIStreamListener listener); */
  NS_IMETHOD AsyncRead(PRUint32 startPosition, PRInt32 readCount, nsISupports *ctxt, nsIStreamListener *listener) = 0;

  /**
     * Writes asynchronously to the URL's specified destination. Notifications
     * are provided to the stream observer on the thread of the specified
     * event queue.
     * The startPosition argument designates the offset in the destination where
     * the data will be written.
     * If the writeCount == -1, then all the available data in the input
     * stream is written.
     */
  /* void asyncWrite (in nsIInputStream fromStream, in unsigned long startPosition, in long writeCount, in nsISupports ctxt, in nsIStreamObserver observer); */
  NS_IMETHOD AsyncWrite(nsIInputStream *fromStream, PRUint32 startPosition, PRInt32 writeCount, nsISupports *ctxt, nsIStreamObserver *observer) = 0;

  /**
     * Load attribute flags. These may be or'd together.
     *
     * Note that more will follow for each protocol's implementation of a channel, 
     * although channel writers have to be careful to not let the flag bits 
     * overlap. Otherwise, users won't be able to create a single flag word
     * of load attributes that applies to a number of different channel types.
     */
  enum { LOAD_NORMAL = 0 };

  enum { LOAD_BACKGROUND = 1 };

  enum { LOAD_DOCUMENT_URI = 2 };

  enum { LOAD_RETARGETED_DOCUMENT_URI = 4 };

  enum { INHIBIT_PERSISTENT_CACHING = 256 };

  enum { FORCE_RELOAD = 512 };

  enum { FORCE_VALIDATION = 1024 };

  enum { CACHE_AS_FILE = 2048 };

  enum { VALIDATE_NEVER = 4096 };

  enum { VALIDATE_ALWAYS = 8192 };

  enum { VALIDATE_ONCE_PER_SESSION = 16384 };

  enum { VALIDATE_HEURISTICALLY = 32768 };

  /**
     * Accesses the load attributes for the channel. E.g. setting the load 
     * attributes with the LOAD_QUIET bit set causes the loading process to
     * not deliver status notifications to the program performing the load,
     * and to not contribute to keeping any nsILoadGroup it may be contained
     * in from firing its OnLoadComplete notification.
     */
  /* attribute nsLoadFlags loadAttributes; */
  NS_IMETHOD GetLoadAttributes(nsLoadFlags *aLoadAttributes) = 0;
  NS_IMETHOD SetLoadAttributes(nsLoadFlags aLoadAttributes) = 0;

  /**
     * Returns the content MIME type of the channel if available. Note that the 
     * content type can often be wrongly specified (wrong file extension, wrong
     * MIME type, wrong document type stored on a server, etc.) and the caller
     * most likely wants to verify with the actual data. 
     */
  /* attribute string contentType; */
  NS_IMETHOD GetContentType(char * *aContentType) = 0;
  NS_IMETHOD SetContentType(const char * aContentType) = 0;

  /**
     * Returns the length of the data assiciated with the channel if available.
     * If the length is unknown then -1 is returned.
     */
  /* readonly attribute long contentLength; */
  NS_IMETHOD GetContentLength(PRInt32 *aContentLength) = 0;

  /**
     * Accesses the owner corresponding to the entity that is 
     * responsible for this channel. Used by security code to grant
     * or diminish privileges to mobile code loaded from this channel. 
     */
  /* attribute nsISupports owner; */
  NS_IMETHOD GetOwner(nsISupports * *aOwner) = 0;
  NS_IMETHOD SetOwner(nsISupports * aOwner) = 0;

  /**
     * Accesses the load group in which the channel is a currently a member.
     */
  /* attribute nsILoadGroup loadGroup; */
  NS_IMETHOD GetLoadGroup(nsILoadGroup * *aLoadGroup) = 0;
  NS_IMETHOD SetLoadGroup(nsILoadGroup * aLoadGroup) = 0;

  /**
     * Accesses the capabilities callbacks of the channel. This is set by clients
     * who wish to provide a means to receive progress, status and protocol-specific 
     * notifications.
     */
  /* attribute nsIInterfaceRequestor notificationCallbacks; */
  NS_IMETHOD GetNotificationCallbacks(nsIInterfaceRequestor * *aNotificationCallbacks) = 0;
  NS_IMETHOD SetNotificationCallbacks(nsIInterfaceRequestor * aNotificationCallbacks) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSICHANNEL \
  NS_IMETHOD GetOriginalURI(nsIURI * *aOriginalURI); \
  NS_IMETHOD GetURI(nsIURI * *aURI); \
  NS_IMETHOD OpenInputStream(PRUint32 startPosition, PRInt32 readCount, nsIInputStream **_retval); \
  NS_IMETHOD OpenOutputStream(PRUint32 startPosition, nsIOutputStream **_retval); \
  NS_IMETHOD AsyncOpen(nsIStreamObserver *observer, nsISupports *ctxt); \
  NS_IMETHOD AsyncRead(PRUint32 startPosition, PRInt32 readCount, nsISupports *ctxt, nsIStreamListener *listener); \
  NS_IMETHOD AsyncWrite(nsIInputStream *fromStream, PRUint32 startPosition, PRInt32 writeCount, nsISupports *ctxt, nsIStreamObserver *observer); \
  NS_IMETHOD GetLoadAttributes(nsLoadFlags *aLoadAttributes); \
  NS_IMETHOD SetLoadAttributes(nsLoadFlags aLoadAttributes); \
  NS_IMETHOD GetContentType(char * *aContentType); \
  NS_IMETHOD SetContentType(const char * aContentType); \
  NS_IMETHOD GetContentLength(PRInt32 *aContentLength); \
  NS_IMETHOD GetOwner(nsISupports * *aOwner); \
  NS_IMETHOD SetOwner(nsISupports * aOwner); \
  NS_IMETHOD GetLoadGroup(nsILoadGroup * *aLoadGroup); \
  NS_IMETHOD SetLoadGroup(nsILoadGroup * aLoadGroup); \
  NS_IMETHOD GetNotificationCallbacks(nsIInterfaceRequestor * *aNotificationCallbacks); \
  NS_IMETHOD SetNotificationCallbacks(nsIInterfaceRequestor * aNotificationCallbacks); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSICHANNEL(_to) \
  NS_IMETHOD GetOriginalURI(nsIURI * *aOriginalURI) { return _to ## GetOriginalURI(aOriginalURI); } \
  NS_IMETHOD GetURI(nsIURI * *aURI) { return _to ## GetURI(aURI); } \
  NS_IMETHOD OpenInputStream(PRUint32 startPosition, PRInt32 readCount, nsIInputStream **_retval) { return _to ## OpenInputStream(startPosition, readCount, _retval); } \
  NS_IMETHOD OpenOutputStream(PRUint32 startPosition, nsIOutputStream **_retval) { return _to ## OpenOutputStream(startPosition, _retval); } \
  NS_IMETHOD AsyncOpen(nsIStreamObserver *observer, nsISupports *ctxt) { return _to ## AsyncOpen(observer, ctxt); } \
  NS_IMETHOD AsyncRead(PRUint32 startPosition, PRInt32 readCount, nsISupports *ctxt, nsIStreamListener *listener) { return _to ## AsyncRead(startPosition, readCount, ctxt, listener); } \
  NS_IMETHOD AsyncWrite(nsIInputStream *fromStream, PRUint32 startPosition, PRInt32 writeCount, nsISupports *ctxt, nsIStreamObserver *observer) { return _to ## AsyncWrite(fromStream, startPosition, writeCount, ctxt, observer); } \
  NS_IMETHOD GetLoadAttributes(nsLoadFlags *aLoadAttributes) { return _to ## GetLoadAttributes(aLoadAttributes); } \
  NS_IMETHOD SetLoadAttributes(nsLoadFlags aLoadAttributes) { return _to ## SetLoadAttributes(aLoadAttributes); } \
  NS_IMETHOD GetContentType(char * *aContentType) { return _to ## GetContentType(aContentType); } \
  NS_IMETHOD SetContentType(const char * aContentType) { return _to ## SetContentType(aContentType); } \
  NS_IMETHOD GetContentLength(PRInt32 *aContentLength) { return _to ## GetContentLength(aContentLength); } \
  NS_IMETHOD GetOwner(nsISupports * *aOwner) { return _to ## GetOwner(aOwner); } \
  NS_IMETHOD SetOwner(nsISupports * aOwner) { return _to ## SetOwner(aOwner); } \
  NS_IMETHOD GetLoadGroup(nsILoadGroup * *aLoadGroup) { return _to ## GetLoadGroup(aLoadGroup); } \
  NS_IMETHOD SetLoadGroup(nsILoadGroup * aLoadGroup) { return _to ## SetLoadGroup(aLoadGroup); } \
  NS_IMETHOD GetNotificationCallbacks(nsIInterfaceRequestor * *aNotificationCallbacks) { return _to ## GetNotificationCallbacks(aNotificationCallbacks); } \
  NS_IMETHOD SetNotificationCallbacks(nsIInterfaceRequestor * aNotificationCallbacks) { return _to ## SetNotificationCallbacks(aNotificationCallbacks); } 


/* starting interface:    nsIInputStreamChannel */
#define NS_IINPUTSTREAMCHANNEL_IID_STR "bfbf843a-8b89-11d3-8cd9-0060b0fc14a3"

#define NS_IINPUTSTREAMCHANNEL_IID \
  {0xbfbf843a, 0x8b89, 0x11d3, \
    { 0x8c, 0xd9, 0x00, 0x60, 0xb0, 0xfc, 0x14, 0xa3 }}

/**
 * nsIInputStreamChannel is an interface that allows for the initialization
 * of a simple nsIChannel that is constructed from a single input stream and
 * associated content type. Input stream channels only allow the input stream
 * to be accessed, not the output stream.
 */
class nsIInputStreamChannel : public nsIChannel {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IINPUTSTREAMCHANNEL_IID)

  /* void init (in nsIURI uri, in string contentType, in long contentLength, in nsIInputStream inStr, in nsILoadGroup aLoadGroup, in nsIInterfaceRequestor notificationCallbacks, in nsLoadFlags loadAttributes, in nsIURI originalURI, in unsigned long bufferSegmentSize, in unsigned long bufferMaxSize); */
  NS_IMETHOD Init(nsIURI *uri, const char *contentType, PRInt32 contentLength, nsIInputStream *inStr, nsILoadGroup *aLoadGroup, nsIInterfaceRequestor *notificationCallbacks, nsLoadFlags loadAttributes, nsIURI *originalURI, PRUint32 bufferSegmentSize, PRUint32 bufferMaxSize) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIINPUTSTREAMCHANNEL \
  NS_IMETHOD Init(nsIURI *uri, const char *contentType, PRInt32 contentLength, nsIInputStream *inStr, nsILoadGroup *aLoadGroup, nsIInterfaceRequestor *notificationCallbacks, nsLoadFlags loadAttributes, nsIURI *originalURI, PRUint32 bufferSegmentSize, PRUint32 bufferMaxSize); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIINPUTSTREAMCHANNEL(_to) \
  NS_IMETHOD Init(nsIURI *uri, const char *contentType, PRInt32 contentLength, nsIInputStream *inStr, nsILoadGroup *aLoadGroup, nsIInterfaceRequestor *notificationCallbacks, nsLoadFlags loadAttributes, nsIURI *originalURI, PRUint32 bufferSegmentSize, PRUint32 bufferMaxSize) { return _to ## Init(uri, contentType, contentLength, inStr, aLoadGroup, notificationCallbacks, loadAttributes, originalURI, bufferSegmentSize, bufferMaxSize); } 

#define NS_INPUTSTREAMCHANNEL_CID                    \
{ /* 436d84f8-8b8a-11d3-8cd9-0060b0fc14a3 */         \
    0x436d84f8,                                      \
    0x8b8a,                                          \
    0x11d3,                                          \
    {0x8c, 0xd9, 0x00, 0x60, 0xb0, 0xfc, 0x14, 0xa3} \
}

/* starting interface:    nsIFileChannel */
#define NS_IFILECHANNEL_IID_STR "6eef6444-c7e3-11d3-8cda-0060b0fc14a3"

#define NS_IFILECHANNEL_IID \
  {0x6eef6444, 0xc7e3, 0x11d3, \
    { 0x8c, 0xda, 0x00, 0x60, 0xb0, 0xfc, 0x14, 0xa3 }}

/**
 * nsIFileChannel is an interface that allows for the initialization
 * of a simple nsIChannel that is constructed from a single nsIFile and
 * associated content type. 
 */
class nsIFileChannel : public nsIChannel {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IFILECHANNEL_IID)

  /* void init (in nsIFile file, in long mode, in string contentType, in long contentLength, in nsILoadGroup aLoadGroup, in nsIInterfaceRequestor notificationCallbacks, in nsLoadFlags loadAttributes, in nsIURI originalURI, in unsigned long bufferSegmentSize, in unsigned long bufferMaxSize); */
  NS_IMETHOD Init(nsIFile *file, PRInt32 mode, const char *contentType, PRInt32 contentLength, nsILoadGroup *aLoadGroup, nsIInterfaceRequestor *notificationCallbacks, nsLoadFlags loadAttributes, nsIURI *originalURI, PRUint32 bufferSegmentSize, PRUint32 bufferMaxSize) = 0;

  /* readonly attribute nsIFile file; */
  NS_IMETHOD GetFile(nsIFile * *aFile) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIFILECHANNEL \
  NS_IMETHOD Init(nsIFile *file, PRInt32 mode, const char *contentType, PRInt32 contentLength, nsILoadGroup *aLoadGroup, nsIInterfaceRequestor *notificationCallbacks, nsLoadFlags loadAttributes, nsIURI *originalURI, PRUint32 bufferSegmentSize, PRUint32 bufferMaxSize); \
  NS_IMETHOD GetFile(nsIFile * *aFile); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIFILECHANNEL(_to) \
  NS_IMETHOD Init(nsIFile *file, PRInt32 mode, const char *contentType, PRInt32 contentLength, nsILoadGroup *aLoadGroup, nsIInterfaceRequestor *notificationCallbacks, nsLoadFlags loadAttributes, nsIURI *originalURI, PRUint32 bufferSegmentSize, PRUint32 bufferMaxSize) { return _to ## Init(file, mode, contentType, contentLength, aLoadGroup, notificationCallbacks, loadAttributes, originalURI, bufferSegmentSize, bufferMaxSize); } \
  NS_IMETHOD GetFile(nsIFile * *aFile) { return _to ## GetFile(aFile); } 

#define NS_FILECHANNEL_CLASSNAME "File Channel"
#define NS_FILECHANNEL_PROGID "component://netscape/network/file-channel"
#define NS_FILECHANNEL_CID                           \
{ /* 7036066e-c7e3-11d3-8cda-0060b0fc14a3 */         \
    0x7036066e,                                      \
    0xc7e3,                                          \
    0x11d3,                                          \
    {0x8c, 0xda, 0x00, 0x60, 0xb0, 0xfc, 0x14, 0xa3} \
}

#endif /* __gen_nsIChannel_h__ */
